/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.pluginmgt;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.tinygroup.command.CommandSystem;
import org.tinygroup.command.config.Commands;
import org.tinygroup.i18n.I18nMessageFactory;
import org.tinygroup.i18n.I18nMessages;
import org.tinygroup.plugin.exception.PluginException;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

/**
 * 命令行的插件执行器
 * 
 * @author luoguo
 * 
 */
public class PluginCommander extends Thread {
	private static final String QUIT_COMMAND = "quit";
	private static final String EXIT_COMMAND = "exit";
	private final String configPath = "PluginCommands.xml";// 加载路径
	CommandSystem commandSystem;// 命令系统
	XStream stream = XStreamFactory.getXStream(CommandSystem.COMMANDS_XSTREAM);
	private I18nMessages i18nMessages = I18nMessageFactory.getI18nMessages();

	public CommandSystem getCommandSystem() {
		return commandSystem;
	}

	public void setStream(XStream stream) {
		this.stream = stream;
	}

	public PluginCommander() {
	}

	/**
	 * 线程运行方法
	 * 
	 * @param args
	 * @throws IOException
	 */
	public void run() {
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		String command = "";
		while (command != null) {
			try {
				commandSystem.indentPrint(">", 0);
				command = in.readLine();
				if (command.equalsIgnoreCase(EXIT_COMMAND)
						|| command.equalsIgnoreCase(QUIT_COMMAND)) {
					commandSystem.println(i18nMessages
							.getMessage("pluginmgt.byebye"));
					return;
				}
				commandSystem.execute(command);
			} catch (IOException e) {
				throw new PluginException(
						i18nMessages.getMessage("pluginmgt.readIOExecption"), e);
			}
		}
	}

	public void init() {

		Commands commands = (Commands) stream.fromXML(PluginCommander.class
				.getClassLoader().getResourceAsStream(configPath));
		commandSystem = CommandSystem.getInstance("cmd", commands, System.out);

	}
}
