/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.pluginmgt.cmd;

import org.tinygroup.command.CommandGoalExecutor;
import org.tinygroup.command.CommandSystem;
import org.tinygroup.context.Context;
import org.tinygroup.plugin.PluginManager;
import org.tinygroup.plugin.config.PluginInfo;
import org.tinygroup.springutil.SpringUtil;

/**
 * 
 * @author luoguo
 *
 */

public abstract class AbstractPluginCommand implements CommandGoalExecutor {
	private String id;
	private String version;
	private int level;

	public int getLevel() {
		return level;
	}

	public void setLevel(int level) {
		this.level = level;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public void execute(CommandSystem commandHelper, Context context) {
		if (level > 0) {
			dealPluginByLevel(commandHelper, context);
		} else if (isNull(id) && isNull(version)) {
			dealAllPlugin(commandHelper, context);
		} else if (isNull(id)) {
			commandHelper.indentPrint("使用参数version时,id不可为空", 0);
			commandHelper.println();
		} else if (isNull(version)) {
			dealPluginById(commandHelper, context);
		} else {
			dealPluginByIdAndVersion(commandHelper, context);
		}
	}

	private void dealAllPlugin(CommandSystem commandHelper, Context context) {
		actionAllPlugin(commandHelper, context);
	}

	private void dealPluginById(CommandSystem commandHelper, Context context) {
		PluginManager pluginManager = SpringUtil.getBean(
				"pluginManager");;
		PluginInfo plugin = pluginManager.getPluginInfo(id);
		if (plugin == null) {
			commandHelper.indentPrint("插件不存在，ID:" + id, 0);
			commandHelper.println();
			return;
		}
		actionPlugin(commandHelper, context, plugin);
	}

	private void dealPluginByIdAndVersion(CommandSystem commandHelper,
			Context context) {
		PluginManager pluginManager = SpringUtil.getBean(
				"pluginManager");
		PluginInfo plugin = pluginManager.getPluginInfo(id, version);
		if (plugin == null) {
			commandHelper.indentPrint("插件不存在，ID:" + id + ",version:" + version,
					0);
			commandHelper.println();
			return;
		}
		actionPlugin(commandHelper, context, plugin);
	}

	private void dealPluginByLevel(CommandSystem commandHelper, Context context) {
		actionPluginByLevel(commandHelper, context, level);
	}

	private boolean isNull(String string) {
		return string == null || "".equals(string);
	}

	protected abstract void actionPluginByLevel(CommandSystem commandHelper,
			Context context, int level);

	protected abstract void actionAllPlugin(CommandSystem commandHelper,
			Context context);

	protected abstract void actionPlugin(CommandSystem commandHelper,
			Context context, PluginInfo plugin);

}
